/*
	ProWesS, PROGS Windowing System

	item support type "iTEM", passive version !

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 18 December 1995, based on item_c
	22.03.1996 adjusted for internal use of pixel values
	06.06.1996 use system default ink when none defined
	28.06.1996 faster (and shorter) border drawing thanks to PFP and better
			   handling of linewidth in PROforma
*/

#include <mem.h>

#include "PWHandler.h"

/* global statusses */

#define p100 short2pt(100)
pt Xpt2pix, Ypt2pix;
pt width, xwidth=pt_one, ywidth=pt_one; /* width nominal and each direction */
ColourRGB ink	= { p100, 0, 0 }; char Kink=FALSE;

static Error DrawBorder(PWObject obj, ColourRGB *colour)
{
	PWRegion *reg=obj->event.region;
	Gstate gstate=reg->system->gstate;
	pt xorg=short2pt(reg->hit_xorg);
	pt yorg=short2pt(reg->hit_yorg);
	pt xsiz=short2pt(reg->hit_xend)-xorg+xwidth;
	pt ysiz=short2pt(reg->hit_yend)-yorg+ywidth;

	PFPathStroked(gstate);
	PFColourRGB(gstate, colour);
	PFLineWidth(gstate, width);

	PFPMoveTo(gstate, xorg-xwidth/2, yorg-ywidth/2);
	PFPRectangle(gstate, xsiz, ysiz);
	PFPathDraw(gstate);

	reg->system->status|=SSTATUS_PAGESHOW;

	return ERR_OK;
}

Error Draw(PWObject object)
{
	return DrawBorder(object,&ink);
}

Error OK(PWObject objetc)
{
	return ERR_OK;
}

Error Define(char *name, char *value)
{
	ColourRGB *colour;
	double red, green, blue, db;

	LinkPROforma();
	PWpoint2pixel(&Xpt2pix,&Ypt2pix);

	if (STRSameCD(name,"ITEM-BORDER-COLOUR"))
	{
		STRFormatGet(value,NULL,"%f %f %f",&red,&green,&blue);
		Kink=TRUE;
		ink.red  =double2pt(red);
		ink.green=double2pt(green);
		ink.blue =double2pt(blue);
		return ERR_OK;
	}
	if (STRSameCD(name,"ITEM-BORDER-WIDTH"))
	{
		STRFormatGet(value,NULL,"%f",&db);
		width=double2pt(db);

		xwidth=pt_round(fixmul(width,Xpt2pix)); if (xwidth<pt_one) xwidth=pt_one;
		ywidth=pt_round(fixmul(width,Ypt2pix)); if (ywidth<pt_one) ywidth=pt_one;
		return ERR_OK;
	}
	return ERR_IPAR;
}

Error DetermineSize(PWObject obj)
{
	PWRegion *region=obj->event.region;

	if (!Kink) PWQuery(obj,PW_DEFAULT_MIDDLEGROUND,&ink);

	region->hit_off_lft=region->hit_off_rgt=pt2short(xwidth)*2;
	region->hit_off_top=region->hit_off_bot=pt2short(ywidth)*2;

	region->tot_xsiz+=4*xwidth;
	region->tot_ysiz+=4*ywidth;
	return ERR_OK;
}

PWTypeDef init={
	PW_TYPEDEF_FLAG, PW_TYPE_iTEM, PW_TYPE_EVENT_REGION,
	NULL, NULL, NULL, NULL, OK, OK, NULL, Define,
	Draw, DetermineSize, NULL
};
