/*
	Progress bar type for Prowess
	copyright (c) W. Lenerz 1997
	versions:
	1.00	1997 Jul 01 23:14:51
*/

#include <mem.h>
#include <ctype.h>
#include "PWHandler.h"
#include "pbar.h"

/* some constants */
#define catch(X) do { if (err=(X)) return err; } while (0)
#define p100 short2pt(100)
#define p1 short2pt(1)
#define ph short2pt(1)/2

/* global statusses */
ColourRGB ink = { 0, p100, 0 }; char Kink=FALSE;    /* ink col for possible text*/
ColourRGB bar = { p100,0,p100}; char Kbar=FALSE;    /* bar colour */
ColourRGB pap = { p100, 0, 0} ; char Kpap=FALSE;    /* paper colour */

char font[PF_MAXFONTNAME]="";		/* name of possible PROforma font to use */
pt fontsize=0;						/* fontsize to use */
short baseline=0;					/* font baseline pos. in pixels (at fontsize) */
short height=0; 					/* height of font (at fontsize), in pixels */
char pctsign[2]={"%"};              /* used if percentage is also printed */
char ohp[5]={"100%"};               /* max string for percentage */
char recalculate=TRUE;				/* flag for recalcuation */

/* loose item object definition */

typedef struct {
	PWBaseObject;					/* basic object, needed for all */
	char *text; 					/* possible text inside bar */
	short textwidth;				/* width of text in pixels if any */
	short minsize;					/* minimum size of bar, in pixels */
	int textlength; 				/* number of chars in text */
	int total;						/* total bar length (in USER units) i.e. 100% */
	int progress;					/* progress up to now as percentage 1 to 100*/
	char instant_redraw;			/* flag for instant redraw */
	char updown;					/* does it go up / down? */
	char showperct; 				/* also print percentage in bar? */
	char percentlast;				/* if combined text & %, print % last */
	char initialised;
} Object;

/* object handling routines */

Error Create(PWObject owner, PWObject *ret, PWEvent event, int *taglist)
{
	Error err=ERR_OK;
	Object *obj;

	LinkPROforma();
	err=PWAllocate(event.region->system,sizeof(Object),ret);
	obj=(Object *)*ret;
	obj->updown=FALSE;		/* just to make sure! */

	/* see if there is a PW_PBAR_UPDOWN tag */
	while (*taglist)
	{
		if (*taglist==PW_PBAR_UPDOWN)
		{
			obj->updown=TRUE;			/* show that bar is up/down */
			*taglist&=PW_TAGMASK_SKIP;	/* mask this tag to be skipped */
		}
		taglist=PWSkipTag(taglist);
	}
	return err;
}

Error Remove(PWObject object)
{
	return PWRelease(object,object);
}

short VertPrint(Gstate gstate,	char *to_print, int number,
								short yorg, short hit_xsiz)
/* this prints a string up/down  - the x origin depends on the width of
   each character, so that they stay nicely centered */
{
	char text[2]={0,0};                     /* used to print each char */
	int i;
	short xorg; 							/* x orgin of char */
	if (!number) return yorg;				/* do nothing if no string length */
	for (i=0; i!=number;i++)
	{
		text[0]=to_print[i];				/* char to print */
		PFPStringWidth (gstate,text,&xorg); /* width of this char */
		xorg=(hit_xsiz-xorg)/2; 			/* x origin for this char */
		PFPMoveTo(gstate,short2pt(xorg),short2pt(yorg));
		PFPStringShow(gstate,text); 		/* print this one char */
		yorg+=height;						/* and move down */
	}
	return yorg;							/* return new yorg */
}

/* now draw the object: first the bar, then the text */
Error Draw(PWObject object)
{
	Object *obj=(Object *)object;
	PWRegion *reg=obj->event.region;
	Gstate gstate=reg->system->gstate;
	short hit_xsiz=reg->hit_xend-reg->hit_xorg;
	short hit_ysiz=reg->hit_yend-reg->hit_yorg;
	pt xsiz=xpix2PF(hit_xsiz)-p1;	/* do not use short2pt */
	pt ysiz=ypix2PF(hit_ysiz);
	pt pbar;

	/* first draw bar by making appropriate subwindow, setting its paper
	colour to the bar colour & clearing it */

	PFPWindowSubSet(gstate,hit_xsiz,hit_ysiz,reg->hit_xorg,reg->hit_yorg);
	PFPaperColourRGB (gstate,&pap);
	PFPageClear (gstate);				/* clear object/bar/text */
	if (!obj->updown)
	{
		pbar=(xsiz/100)*obj->progress;
		if (pbar>xsiz || obj->progress==100) pbar=xsiz-ph;
		PFWindowSubSet(gstate,pbar,ysiz,ph,0);	/* set window to this size */
	}
	else
	{
		pbar=(ysiz/100)*obj->progress;
		if (pbar>ysiz || obj->progress==100) pbar=ysiz;
		PFWindowSubSet(gstate,xsiz-p1,pbar,ph,ysiz-pbar);
	}
	PFPaperColourRGB (gstate,&bar); 	/* colour to draw bar in */
	PFPageClear (gstate);				/* this "draws " the bar */
	PFWindowSubRestore (gstate);		/* restore window */

	/* now print possible string or percentage, or combination of both */
	if (obj->text || obj->showperct)
	{
		char percentage[5]={0,0,0,0,0}; /* string for percentage */
		PFColourRGB (gstate, &ink); 	/* colour for string */
		PFFontLoad(gstate,font);
		PFFontScale(gstate,fontsize);	/* make sure we have the right font etc */
		if (obj->showperct)
		{
			STRFormatPut(percentage,"%d",obj->progress);
			STRAppend (pctsign,percentage); /* make the percent string */
		}
		if (!obj->updown)
		{
			/* first determine where to print */
			pt xorg=0,yorg=0;
			short width=0;
			if (obj->showperct)
			{
				PFPStringWidth (gstate,percentage,&width); /* width of percentage string */
			}
			if (obj->text)
			{
				width+=obj->textwidth;	/* + width of text string */
			}
			xorg=short2pt((hit_xsiz-width)/2);	/* xorigin for drawing */
			if (xorg<0) xorg=0; 				/* may happen if text changed */
			yorg=short2pt((hit_ysiz-height)/2+baseline);
			PFPMoveTo(gstate,xorg,yorg);

			/* now print */
			if (obj->percentlast)
			{
				PFPStringShow(gstate,obj->text);	/* print text first */
				PFPStringShow(gstate,percentage);
			}
			else
			{
				PFPStringShow(gstate,percentage);	/* print perentage first */
				PFPStringShow(gstate,obj->text);
			}
		}
		else
		{                                   /* print up/down */
			short yorg=0;
			int tlen=0;
			/* first get the y org */
			if (obj->showperct)
			{
				STRLength (percentage,&tlen);
				yorg=tlen*height;			/* height of this string */
			}
			if (obj->text)
			{
				yorg+=obj->textlength*height;
			}

			yorg=(hit_ysiz-yorg)/2 + baseline;	/* make y origin */
			if (obj->percentlast)
			{
				yorg=VertPrint(gstate,	obj->text,	obj->textlength,
										yorg, hit_xsiz);
				yorg=VertPrint(gstate,	percentage, tlen,
										yorg, hit_xsiz);
			}
			else
			{
				yorg=VertPrint(gstate,	percentage, tlen,
										yorg, hit_xsiz);
				yorg=VertPrint(gstate,	obj->text,	obj->textlength,
										yorg, hit_xsiz);
			}
		}
	}
	if (obj->instant_redraw)
	{
		PFPageShow (gstate);   /* redraw immediately */
		obj->instant_redraw=FALSE;
	}
	PFWindowSubRestore(gstate);
	return ERR_OK;
}

Error Redraw_now(PWObject object)
{
	Object *obj=(Object *)object;
	obj->instant_redraw=TRUE;
	return Draw (object);
}

Error Change(PWObject object, int *taglist)
{
	Object *obj=(Object *)object;
	PWRegion *region=obj->event.region;
	Gstate gstate=region->system->gstate;
	PWSystem *system=obj->event.region->system;
	Error err=ERR_OK;
	char unrecognised_tag=FALSE;
	while (*taglist)
	{
		switch (*taglist)
		{
		case PW_PBAR_TEXT:				/* set text inside bar */
			PFFontLoad(gstate,font);	/* get font */
			PFFontScale(gstate,fontsize);/* scale to size */
			obj->text=(char *)(*(taglist+1));
			PFPStringWidth(gstate,obj->text,&obj->textwidth); /* get string windth */
			STRLength (obj->text,&obj->textlength);
			system->status|=SSTATUS_WINCHANGE; /* me must make sure the text fits */
			break;

		case PW_PBAR_TOTAL: 			/* set total i.e. 100% in user units */
			obj->total=(*(taglist+1));
			break;

		case PW_PBAR_PROGRESS:			/* set progress in user units*/
			obj->progress=((*(taglist+1))*100)/obj->total; /* mk percentage */
			goto prog;

		case PW_PBAR_PROGRESS_PERCENT:	/* set progress in percent */
			obj->progress=(*(taglist+1));
prog:		if (obj->progress>100) obj->progress=100;
			if (obj->progress<0) obj->progress=0;
			if (obj->initialised) Redraw_now(object);
			break;

		case PW_PBAR_SHOW_PERCENT:
			obj->showperct=(*(taglist+1));
			break;

		case PW_PBAR_PERCENT_LAST:
			obj->percentlast=(*(taglist+1));
			break;
		case PW_PBAR_MINSIZE:
			obj->minsize=(short)*(taglist+1);
			break;
		case PW_PBAR_MINSIZE_TEXT:
			PFFontLoad(gstate,font);	/* get font */
			PFFontScale(gstate,fontsize);/* scale to size */
			if (!obj->updown)
				PFPStringWidth(gstate,(char *)(*(taglist+1)),&obj->minsize);
			else
			{
				int t;
				PBbox bbox;
				PFPFontBbox(gstate,0,&bbox);	/* get max size of biggest char */
				STRLength ((char *)(*(taglist+1)),&t);
				obj->minsize=t*bbox.ysiz;
			}
			break;
		default:
			if (*taglist&(~PW_TAGMASK_SKIP))
				unrecognised_tag=TRUE;
			break;
		}
		taglist=PWSkipTag(taglist); 		/* get next tag now */
	}
	if (unrecognised_tag) err=ERR_IPAR;
	return err;
}

Error DetermineSize(PWObject object)
{
	Object *obj=(Object *)object;
	PWRegion *region=obj->event.region;
	Gstate gstate=region->system->gstate;
	short width=20,high=5;			   /* min width and height in pixels */
	PBbox bbox;
	region->hit_off_lft=region->hit_off_rgt=
	region->hit_off_top=region->hit_off_bot=0;
	if (recalculate)
	{
		if (!Kink) PWQuery(object,PW_DEFAULT_FOREGROUND,&ink);
		if (!Kpap) PWQuery(object,PW_DEFAULT_BACKGROUND,&pap);
		if (!Kbar) PWQuery(object,PW_DEFAULT_MIDDLEGROUND,&bar);
		if (!fontsize) PWQuery(object,PW_DEFAULT_FONTSIZE,&fontsize);
		if (!*font) PWQuery(object,PW_DEFAULT_FONT,font);
		recalculate=FALSE;
	}
	if (!gstate) return ERR_OK; 	/* oops */
	PFFontLoad(gstate,font);		/* get font */
	PFFontScale(gstate,fontsize);	/* scale to size */
	PFPFontBbox(gstate,0,&bbox);	/* get max size of biggest char */
	baseline=bbox.yorg; 			/* baseline of char */
	height=bbox.ysiz;				/* max height of char */

	high=height;					/* minimum y size ! */

	/* make sure size can hold text(s), if any */
	if (obj->text || obj->showperct)
	{

		width=0;
		if (!obj->updown)
		{
			if (obj->showperct)
			{
				PFPStringWidth(gstate,ohp,&width);	/* get width of "100%" */
			}
			if (obj->text)
			{
				PFPStringWidth(gstate,obj->text,&obj->textwidth); /* get string windth */
				width+=obj->textwidth;	/* possible combined width of % and str */
			}
			width+=8;					/* leave some pixels around! */
			if (width<obj->minsize)
				width=obj->minsize;
		}
		else
		{
			int slen;
			width=bbox.xsiz+8;			/* max x size of one char */
			high=0;
			if (obj->showperct)
			{
				STRLength (ohp,&slen);
				high=slen*height;		/* we need that much space */
			}
			if (obj->text)
			{
				high+=obj->textlength*height;  /* + possible text */
			}
			high+=8;
			if (high<obj->minsize)
				high=obj->minsize;
		}
	}
	region->tot_ysiz=short2pt(high);
	region->tot_xsiz=short2pt(width);
	obj->initialised=TRUE;
	return ERR_OK;
}

Defines defines[]={
{ "INK-COLOUR", PW_DEFINE_COLOUR,   NULL,   &ink,       &Kink },
{ "BAR-COLOUR", PW_DEFINE_COLOUR,   NULL,   &bar,       &Kbar },
{ "PAPER-COLOUR", PW_DEFINE_COLOUR, NULL,   &pap,       &Kpap },
{ "FONT",       PW_DEFINE_FONT,     NULL,   font,       &recalculate },
{ "FONTSIZE",   PW_DEFINE_PT,       NULL,   &fontsize,  &recalculate },
{ 0 }
};

Error Define(char *name, char *value)
{
	return PWHandleDefine("PBAR-",defines,name,value);
}

PWTypeDef init={
	PW_TYPEDEF_FLAG, PW_TYPE_PBAR, PW_TYPE_EVENT_REGION,
	&Create, &Change, NULL, &Remove, NULL, NULL, NULL, &Define,
	&Draw, &DetermineSize, NULL
};
